@@ -8,7 +8,7 @@
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -48,6 +48,7 @@
 #include <list>
 #include <cmath>
 #include <cstring>
+#include <sstream>
 
 using namespace gcu;
 using namespace std;
@@ -712,13 +713,6 @@ bool Fragment::Load (xmlNodePtr node)
 				m_Atom->AddBond (pBond);
 			}
 			delete pOldAtom;
-			char id[8];
-			int j = 1;
-			id[0] = 'a';
-			do
-				snprintf (id+1, 7, "%d", j++);
-			while (pDoc->GetDescendant (id) != NULL);
-			m_Atom->SetId (id);
 			AddChild (m_Atom);
 			if (!m_Atom->Load (child))
 				return false;
@@ -1189,7 +1183,7 @@ void Fragment::Move (double x, double y, double z)
 
 void Fragment::OnChangeAtom ()
 {
-	if (m_bLoading)
+	if (m_bLoading || !m_Atom)
 		return;
 	Document *pDoc = (Document*) GetDocument ();
 	if (!pDoc) return;
@@ -1197,6 +1191,7 @@ void Fragment::OnChangeAtom ()
 	m_TextItem->ReplaceText (sym, m_BeginAtom, m_EndAtom - m_BeginAtom);
 	m_EndAtom = m_BeginAtom + sym.length ();
 	// FIXME: we probably need to insert a tag there
+	m_StartSel = m_EndSel = m_TextItem->GetCursorPosition ();
 	OnChanged (false);
 }
 
@@ -1415,7 +1410,8 @@ bool Fragment::SetProperty (unsigned property, char const *value)
 	m_bLoading = true;
 	switch (property) {
 	case GCU_PROP_POS2D: {
-		sscanf (value, "%lg %lg", &m_x, &m_y);
+		std::istringstream str (value);
+		str >> m_x >> m_y;
 		gcu::Document *doc = GetDocument ();
 		if (doc) {
 			m_x *= doc->GetScale ();
