@@ -8,7 +8,7 @@
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -286,6 +286,7 @@ void View::OnDeleteSelection (GtkWidget* w)
 			pObject = m_pDoc->GetDescendant ((*k).c_str ());
 			if (!pObject)
 				continue;
+			pObject->EmitSignal (OnChangedSignal);
 			Group = pObject->GetGroup ();
 			ModifiedObjects.insert ((Group)? Group->GetId (): *k);
 		}
@@ -793,7 +794,7 @@ void View::ExportImage (string const &filename, const char* type, int resolution
 		cairo_destroy (cr);
 		g_object_unref (output);
 	} else {
-		GdkPixbuf *pixbuf = BuildPixbuf (resolution);
+		GdkPixbuf *pixbuf = BuildPixbuf (resolution, strcmp (type, "bmp"));
 		GFile *file = g_vfs_get_file_for_uri (g_vfs_get_default (), filename.c_str ());
 		GError *error = NULL;
 		GFileOutputStream *output = g_file_create (file, G_FILE_CREATE_NONE, NULL, &error);
@@ -822,6 +823,7 @@ char *View::BuildSVG ()
 	Render (cr);
 	cairo_destroy (cr);
 	m_pData->ShowSelection (true);
+	gsf_output_write (output, 1, reinterpret_cast < guint8 const * > (""));
 	char *m = g_strdup (reinterpret_cast <char const*> (gsf_output_memory_get_bytes (reinterpret_cast <GsfOutputMemory *> (output))));
 	g_object_unref (output);
 	return m;
@@ -841,6 +843,7 @@ char *View::BuildEPS ()
 	Render (cr);
 	cairo_destroy (cr);
 	m_pData->ShowSelection (true);
+	gsf_output_write (output, 1, reinterpret_cast < guint8 const * > (""));
 	char *m = g_strdup (reinterpret_cast <char const*> (gsf_output_memory_get_bytes (reinterpret_cast <GsfOutputMemory *> (output))));
 	g_object_unref (output);
 	return m;
@@ -851,7 +854,7 @@ static void destroy_surface (G_GNUC_UNUSED guchar *pixels, gpointer data)
 	cairo_surface_destroy (reinterpret_cast <cairo_surface_t *> (data));
 }
 
-GdkPixbuf *View::BuildPixbuf (int resolution)
+GdkPixbuf *View::BuildPixbuf (int resolution, bool transparent)
 {
 	gccv::Rect rect;
 	m_pData->GetObjectBounds (m_pDoc, &rect);
@@ -867,7 +870,7 @@ GdkPixbuf *View::BuildPixbuf (int resolution)
 		zoom = 1.;
 	cairo_surface_t *surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, w, h);
 	cairo_t *cr = cairo_create (surface);
-	if (m_pDoc->GetApp () && !m_pDoc->GetApp ()->GetTransparentBackground ()) {
+	if (!transparent || (m_pDoc->GetApp () && !m_pDoc->GetApp ()->GetTransparentBackground ())) {
 		cairo_set_source_rgb (cr, 1., 1., 1.);
 		cairo_paint (cr);
 	}
@@ -888,8 +891,10 @@ void View::EnsureSize ()
 	m_Canvas->GetRoot ()->GetBounds (x1, y1, x2, y2);
 	if (x1 < 0.) x2 -= x1;
 	if (y1 < 0.) y2 -= y1;
-	if (x2 <= 0. || y2 <= 0.)
+	if (x2 <= 0. || y2 <= 0.) {
+		gtk_widget_set_size_request (m_pWidget, -1, -1);
 		return;
+	}
 	if ((x2 != m_width) || (y2 != m_height)) {
 		m_width = x2;
 		m_height = y2;
