@@ -33,6 +33,7 @@
 #include <gcp/theme.h>
 #include <gcp/view.h>
 #include <gcp/window.h>
+#include <gcugtk/message.h>
 #include <gcugtk/ui-manager.h>
 #include <gccv/canvas.h>
 #include <gccv/group.h>
@@ -287,6 +288,10 @@ void gcpSelectionTool::AddSelection (gcp::WidgetData* data)
 				((*i++)->GetType () == MoleculeType) &&
 				((*i)->GetType () == MoleculeType)));
 		}
+	} else {
+		if (m_UIManager)
+			gtk_widget_set_sensitive (m_MergeBtn, false);
+		win->ActivateActionWidget ("/MainMenu/EditMenu/Erase", false);
 	}
 }
 
@@ -400,11 +405,9 @@ void gcpSelectionTool::CreateGroup ()
 	catch (invalid_argument& e) {
 			pDoc->AbortOperation ();
 			delete pObj;
-			GtkWidget* message = gtk_message_dialog_new (NULL, (GtkDialogFlags) 0,
-								GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, e.what ());
-			gtk_window_set_icon_name (GTK_WINDOW (message), "gchempaint");
-			g_signal_connect_swapped (G_OBJECT (message), "response", G_CALLBACK (gtk_widget_destroy), G_OBJECT (message));
-			gtk_widget_show (message);
+			gcugtk::Message *box = new gcugtk::Message (static_cast < gcugtk::Application * > (pDoc->GetApp ()),
+			                                            e.what (), GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, pDoc->GetGtkWindow ());
+			box->Show ();
 	}
 }
 
@@ -417,12 +420,14 @@ bool gcpSelectionTool::OnRightButtonClicked (gcu::UIManager *UIManager)
 		gtk_action_group_add_action (group, action);
 		m_uiIds.push_front (gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menuitem action='group'/></popup></ui>", -1, NULL));
 		g_signal_connect_swapped (action, "activate", G_CALLBACK (on_group), this);
-		set<TypeId> possible_types, types, wrong_types;
+		set<TypeId> possible_types, types, wrong_types, children_types;
 		set < Object * >::iterator  i = m_pData->SelectedObjects.begin (),
 												end = m_pData->SelectedObjects.end ();
+		children_types.insert ((*i)->GetType());
 		(*i)->GetPossibleAncestorTypes (possible_types);
-		set<TypeId>::iterator type;
+		set<TypeId>::iterator requested, end_requested, type, end_type;
 		for (i++; i != end; i++) {
+			children_types.insert ((*i)->GetType());
 			(*i)->GetPossibleAncestorTypes (types);
 			for (type = possible_types.begin(); type != possible_types.end (); type++)
 				if (types.find (*type) == types.end ())
@@ -432,6 +437,27 @@ bool gcpSelectionTool::OnRightButtonClicked (gcu::UIManager *UIManager)
 			wrong_types.clear ();
 			types.clear ();
 		}
+		/* verify that all requested children are there */
+		end_type = possible_types.end ();
+		for  (type = possible_types.begin (); type != end_type; type++) {
+			TypeDesc const *desc = m_pApp->GetTypeDescription (*type), *rdesc;
+			if (desc->RequiredParents.size () > 0) {
+				wrong_types.insert (*type);
+				continue;
+			}
+			end_requested = desc->RequiredChildren.end ();
+			for (requested = desc->RequiredChildren.begin (); requested != end_requested; requested++) {
+				rdesc = m_pApp->GetTypeDescription (*requested);
+				if (rdesc->RequiredChildren.size () > 0 || rdesc->RequiredParents.size () > 0)
+					continue;
+				if (children_types.find (*requested) == children_types.end()) {
+					wrong_types.insert (*type);
+					break;
+				 }
+			}
+		}
+		for (type = wrong_types.begin(); type != wrong_types.end (); type++)
+			possible_types.erase (*type);
 		if (possible_types.size () == 1) {
 			// Add a new action.
 			m_Type = *possible_types.begin ();
